@@ -186,14 +186,14 @@ rsvg_css_param_arg_offset (const char *str)
 }
 
 static gint
-rsvg_css_clip_rgb_percent (gint in_percent)
+rsvg_css_clip_rgb_percent (gdouble in_percent)
 {
 	/* spec says to clip these values */
-	if (in_percent > 100)
+	if (in_percent > 100.)
 		return 255;
-	else if (in_percent <= 0)
+	else if (in_percent <= 0.)
 		return 0;	
-	return (gint)floor(255. * (double)in_percent / 100. + 0.5);
+	return (gint)floor(255. * in_percent / 100. + 0.5);
 }
 
 static gint
@@ -230,7 +230,7 @@ rsvg_css_color_compare (const void * a, const void * b)
  * Parse a CSS2 color specifier, return RGB value
  */
 guint32
-rsvg_css_parse_color (const char *str)
+rsvg_css_parse_color (const char *str, guint32 inherit)
 {
 	gint val = 0;
 	
@@ -267,15 +267,28 @@ rsvg_css_parse_color (const char *str)
 			
 			if (strstr (str, "%") != 0)
 				{
-					/* assume "rgb (r%, g%, b%)" */
-					if (3 == sscanf (str, " rgb ( %d %% , %d %% , %d %% ) ", &r, &g, &b))
+					gint i, nb_toks;
+					char ** toks;
+
+					/* assume rgb (9%, 100%, 23%) */
+					for(i = 0; str[i] != '('; i++)
+						;
+
+					i++;
+
+					toks = rsvg_css_parse_list(str + i, &nb_toks);
+
+					if (toks)
 						{
-							r = rsvg_css_clip_rgb_percent (r);
-							g = rsvg_css_clip_rgb_percent (g);
-							b = rsvg_css_clip_rgb_percent (b);
+							if (nb_toks == 3)
+								{
+									r = rsvg_css_clip_rgb_percent (g_ascii_strtod (toks[0], NULL));
+									g = rsvg_css_clip_rgb_percent (g_ascii_strtod (toks[1], NULL));
+									b = rsvg_css_clip_rgb_percent (g_ascii_strtod (toks[2], NULL));
+								}
+
+							g_strfreev (toks);
 						}
-					else
-						r = g = b = 0;
 				}
 			else
 				{
@@ -292,7 +305,9 @@ rsvg_css_parse_color (const char *str)
 			
 			val = PACK_RGB (r,g,b);
 		}
-	else
+	else if (!strcmp (str, "inherit"))
+		val = inherit;
+	else 
 		{
 			const static ColorPair color_list [] =
 				{
