@@ -165,6 +165,7 @@ rsvg_paint_server_lin_grad_render (RsvgPaintServer *self, ArtRender *ar,
 			z->agl = agl;
 		}
 
+
 	if (rlg->obj_bbox) {
 		affine[0] = ctx->x1 - ctx->x0;
 		affine[1] = 0.;		
@@ -172,13 +173,14 @@ rsvg_paint_server_lin_grad_render (RsvgPaintServer *self, ArtRender *ar,
 		affine[3] = ctx->y1 - ctx->y0;
 		affine[4] = ctx->x0;
 		affine[5] = ctx->y0;
+
 	} else {
 		for (i = 0; i < 6; i++)
 			affine[i] = ctx->affine[i];
 	}
 
 	art_affine_multiply(affine, rlg->affine, affine);
-	
+
 	/* compute [xy][12] in pixel space */
 	x1 = rlg->x1 * affine[0] + rlg->y1 * affine[2] + affine[4];
 	y1 = rlg->x1 * affine[1] + rlg->y1 * affine[3] + affine[5];
@@ -307,12 +309,14 @@ rsvg_paint_server_rad_grad (RsvgRadialGradient *gradient)
  * Return value: The newly created paint server, or NULL on error.
  **/
 RsvgPaintServer *
-rsvg_paint_server_parse (const RsvgDefs *defs, const char *str)
+rsvg_paint_server_parse (RsvgPaintServer * current, const RsvgDefs *defs, const char *str,
+						 guint32 current_color)
 {
 	guint32 rgb;
 	
 	if (!strcmp (str, "none"))
 		return NULL;
+
 	if (!strncmp (str, "url(", 4))
 		{
 			const char *p = str + 4;
@@ -343,9 +347,18 @@ rsvg_paint_server_parse (const RsvgDefs *defs, const char *str)
 					return NULL;
 				}
 		}
-  else
+	else if (current && !strcmp (str, "inherit"))
+		{
+			rsvg_paint_server_ref (current);
+			return current;
+		}
+	else
 	  {
-		  rgb = rsvg_css_parse_color (str);
+		  if (!strcmp (str, "currentColor"))
+			  rgb = current_color;
+		  else
+			  rgb = rsvg_css_parse_color (str, 0);
+
 		  return rsvg_paint_server_solid (rgb);
 	  }
 }
