@@ -41,6 +41,11 @@ typedef struct _RsvgDefs RsvgDefs;
 typedef struct _RsvgDefVal RsvgDefVal;
 typedef struct _RsvgFilter RsvgFilter;
 
+/* prepare for gettext */
+#ifndef _
+#define _(X) X
+#endif
+
 struct RsvgSaxHandler {
 	void (*free) (RsvgSaxHandler *self);
 	void (*start_element) (RsvgSaxHandler *self, const xmlChar *name, RsvgPropertyBag *atts);
@@ -60,7 +65,7 @@ struct RsvgHandle {
 	int n_state_max;
 	
 	RsvgDefs *defs;
-	gboolean in_defs;
+	guint in_defs;
 	void *current_defs_group;
 
 	GHashTable *css_props;
@@ -119,6 +124,10 @@ typedef enum {
 	RSVG_SIZE_ZOOM_MAX
 } RsvgSizeType;
 
+typedef enum {
+	objectBoundingBox, userSpaceOnUse
+} RsvgCoordUnits;
+
 struct RsvgSizeCallbackData
 {
 	RsvgSizeType type;
@@ -126,6 +135,8 @@ struct RsvgSizeCallbackData
 	double y_zoom;
 	gint width;
 	gint height;
+
+	gboolean keep_aspect_ratio;
 };
 
 /* private */
