@@ -24,6 +24,7 @@
 */
 #include <string.h>
 #include <math.h>
+#include <errno.h>
 
 #include "rsvg-private.h"
 #include "rsvg-styles.h"
@@ -46,26 +47,6 @@
 #define RSVG_ARC_MAGIC ((double) 0.5522847498)
 
 
-typedef struct _RsvgDefsDrawablePath RsvgDefsDrawablePath;
-typedef struct _RsvgDefsDrawableGroup RsvgDefsDrawableGroup;
-typedef struct _RsvgDefsDrawableUse RsvgDefsDrawableUse;
-
-struct _RsvgDefsDrawablePath {
- 	RsvgDefsDrawable super;
- 	char       *d;
-};
-
-struct _RsvgDefsDrawableGroup {
- 	RsvgDefsDrawable super;
- 	GPtrArray *children;
-};
-
-struct _RsvgDefsDrawableUse {
- 	RsvgDefsDrawable super;
- 	RsvgDefsDrawable *child;
-};
-
-
 /**
  * rsvg_close_vpath: Close a vector path.
  * @src: Source vector path.
@@ -257,7 +238,7 @@ rsvg_render_bpath (RsvgHandle *ctx, const ArtBpath *bpath)
 	art_free (affine_bpath);
 	
 	need_tmpbuf = ((state->fill != NULL) && (state->stroke != NULL) &&
-				   state->opacity != 0xff) || state->filter;
+				   state->opacity != 0xff) || state->filter || state->mask;
 	
 	if (need_tmpbuf)
 		rsvg_push_discrete_layer (ctx);
@@ -341,9 +322,10 @@ rsvg_render_path(RsvgHandle *ctx, const char *d)
 }
 
 void 
-rsvg_defs_drawable_draw (RsvgDefsDrawable * self, RsvgHandle *ctx)
+rsvg_defs_drawable_draw (RsvgDefsDrawable * self, RsvgHandle *ctx,
+						 int dominate)
 {
-	self->draw(self, ctx);
+	self->draw(self, ctx, dominate);
 }
 
 static void 
@@ -355,7 +337,8 @@ rsvg_defs_drawable_path_free (RsvgDefVal *self)
 }
 
 static void 
-rsvg_defs_drawable_path_draw (RsvgDefsDrawable * self, RsvgHandle *ctx)
+rsvg_defs_drawable_path_draw (RsvgDefsDrawable * self, RsvgHandle *ctx, 
+							  int dominate)
 {
 	RsvgState *state = rsvg_state_current (ctx);
 	RsvgDefsDrawablePath *path = (RsvgDefsDrawablePath*)self;
@@ -364,16 +347,16 @@ rsvg_defs_drawable_path_draw (RsvgDefsDrawable * self, RsvgHandle *ctx)
 
 	rsvg_state_clone (state, &self->state);
 	if (ctx->n_state > 1)
-		rsvg_state_reinherit(state, &ctx->state[ctx->n_state - 2]);
-	
-	if (state->opacity != 0xff || state->filter)
-		rsvg_push_discrete_layer (ctx);
-	
+		{
+			if (dominate)
+				rsvg_state_dominate(state, &ctx->state[ctx->n_state - 2]);
+			else
+				rsvg_state_reinherit(state, &ctx->state[ctx->n_state - 2]);
+		}
+
 	/* always want to render inside of a <use/> */
 	rsvg_render_path (ctx, path->d);
 	
-	if (state->opacity != 0xff || state->filter)
-		rsvg_pop_discrete_layer (ctx);
 }
 
 static void 
@@ -386,7 +369,8 @@ rsvg_defs_drawable_group_free (RsvgDefVal *self)
 }
 
 static void 
-rsvg_defs_drawable_group_draw (RsvgDefsDrawable * self, RsvgHandle *ctx)
+rsvg_defs_drawable_group_draw (RsvgDefsDrawable * self, RsvgHandle *ctx, 
+							  int dominate)
 {
 	RsvgState *state = rsvg_state_current (ctx);
 	RsvgDefsDrawableGroup *group = (RsvgDefsDrawableGroup*)self;
@@ -395,10 +379,14 @@ rsvg_defs_drawable_group_draw (RsvgDefsDrawable * self, RsvgHandle *ctx)
 	/* combine state definitions */
 	rsvg_state_clone (state, &self->state);
 	if (ctx->n_state > 1)
-		rsvg_state_reinherit(state, &ctx->state[ctx->n_state - 2]);
-	
-	if (state->opacity != 0xff || state->filter)
-		rsvg_push_discrete_layer (ctx);
+		{
+			if (dominate)
+				rsvg_state_dominate(state, &ctx->state[ctx->n_state - 2]);
+			else
+				rsvg_state_reinherit(state, &ctx->state[ctx->n_state - 2]);
+		}
+
+	rsvg_push_discrete_layer (ctx);
 
 	for (i = 0; i < group->children->len; i++)
 		{
@@ -413,16 +401,15 @@ rsvg_defs_drawable_group_draw (RsvgDefsDrawable * self, RsvgHandle *ctx)
 				rsvg_state_init (ctx->state);
 			ctx->n_state++;
 			
-			rsvg_defs_drawable_draw (g_ptr_array_index(group->children, i), ctx);
+			rsvg_defs_drawable_draw (g_ptr_array_index(group->children, i), 
+									 ctx, 0);
 	
 			/* pop the state stack */
 			ctx->n_state--;
 			rsvg_state_finalize (&ctx->state[ctx->n_state]);
 		}			
 
-	if (state->opacity != 0xff || state->filter)
-		rsvg_pop_discrete_layer (ctx);
-	
+	rsvg_pop_discrete_layer (ctx);
 }
 
 static void 
@@ -434,7 +421,8 @@ rsvg_defs_drawable_use_free (RsvgDefVal *self)
 }
 
 static void 
-rsvg_defs_drawable_use_draw (RsvgDefsDrawable * self, RsvgHandle *ctx)
+rsvg_defs_drawable_use_draw (RsvgDefsDrawable * self, RsvgHandle *ctx, 
+							  int dominate)
 {
 	RsvgState *state = rsvg_state_current (ctx);
 	RsvgDefsDrawableUse *use = (RsvgDefsDrawableUse*)self;
@@ -443,7 +431,12 @@ rsvg_defs_drawable_use_draw (RsvgDefsDrawable * self, RsvgHandle *ctx)
 	
 	rsvg_state_clone (state, &self->state);
 	if (ctx->n_state > 1)
-		rsvg_state_reinherit(state, &ctx->state[ctx->n_state - 2]);
+		{
+			if (dominate)
+				rsvg_state_dominate(state, &ctx->state[ctx->n_state - 2]);
+			else
+				rsvg_state_reinherit(state, &ctx->state[ctx->n_state - 2]);
+		}	
 	
 	if (state->opacity != 0xff || state->filter)
 		rsvg_push_discrete_layer (ctx);
@@ -460,7 +453,7 @@ rsvg_defs_drawable_use_draw (RsvgDefsDrawable * self, RsvgHandle *ctx)
 		rsvg_state_init (ctx->state);
 	ctx->n_state++;
 	
-	rsvg_defs_drawable_draw (use->child, ctx);
+	rsvg_defs_drawable_draw (use->child, ctx, 1);
 	
 	/* pop the state stack */
 	ctx->n_state--;
@@ -492,8 +485,7 @@ rsvg_push_def_group (RsvgHandle *ctx, const char * id)
 	group->super.super.free = rsvg_defs_drawable_group_free;
 	group->super.draw = rsvg_defs_drawable_group_draw;
 
-	if (id != NULL)
-		rsvg_defs_set (ctx->defs, id, &group->super.super);
+	rsvg_defs_set (ctx->defs, id, &group->super.super);
 
 	group->super.parent = (RsvgDefsDrawable *)ctx->current_defs_group;
 	if (group->super.parent != NULL)
@@ -520,6 +512,7 @@ rsvg_handle_path (RsvgHandle *ctx, const char * d, const char * id)
 	if (!ctx->in_defs)
 		rsvg_render_path (ctx, d);
 	else {
+	   
 		RsvgDefsDrawablePath *path;
 
 		path = g_new (RsvgDefsDrawablePath, 1);
@@ -528,8 +521,7 @@ rsvg_handle_path (RsvgHandle *ctx, const char * d, const char * id)
 		path->super.super.type = RSVG_DEF_PATH;
 		path->super.super.free = rsvg_defs_drawable_path_free;
 		path->super.draw = rsvg_defs_drawable_path_draw;
-		if (id != NULL)
-			rsvg_defs_set (ctx->defs, id, &path->super.super);
+		rsvg_defs_set (ctx->defs, id, &path->super.super);
 
 		path->super.parent = (RsvgDefsDrawable *)ctx->current_defs_group;
 		if (path->super.parent != NULL)
@@ -749,7 +741,6 @@ rsvg_start_rect (RsvgHandle *ctx, RsvgPropertyBag *atts)
 	y += .01;
 	
 	/* emulate a rect using a path */
-
 	d = g_string_new ("M ");
 	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), x + rx));
 	g_string_append_c (d, ' ');
@@ -827,6 +818,8 @@ rsvg_start_rect (RsvgHandle *ctx, RsvgPropertyBag *atts)
 	g_string_append_c (d, ' ');
 	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), y));
 
+	g_string_append (d, " Z");
+
 	rsvg_handle_path (ctx, d->str, id);
 	g_string_free (d, TRUE);
 }
@@ -1025,6 +1018,362 @@ rsvg_start_ellipse (RsvgHandle *ctx, RsvgPropertyBag *atts)
 	g_string_free (d, TRUE);
 }
 
+static void
+size_prepared_cb (GdkPixbufLoader *loader, 
+				  int              width,
+				  int              height,
+				  gpointer         data)
+{
+	struct {
+		int width;
+		int height;
+		gboolean keep_aspect_ratio;
+	} *info = data;
+	
+	if (info->keep_aspect_ratio) 
+		{
+			if (width < 0)
+				width = 512;
+			if (height < 0)
+				height = 512;
+			
+			if ((double)height * (double)info->width >
+				(double)width * (double)info->height) 
+				{
+					width = 0.5 + (double)width * (double)info->height / (double)height;
+					height = info->height;
+				} 
+			else 
+				{
+					height = 0.5 + (double)height * (double)info->width / (double)width;
+					width = info->width;
+				}
+		} 
+	else 
+		{
+			width = info->width;
+			height = info->height;
+		}
+	
+	gdk_pixbuf_loader_set_size (loader, width, height);
+}
+
+static const char s_UTF8_B64Alphabet[64] = {
+	0x41, 0x42, 0x43, 0x44, 0x45, 0x46, 0x47, 0x48, 0x49, 0x4a, 0x4b, 0x4c, 0x4d, 0x4e, 0x4f,
+	0x50, 0x51, 0x52, 0x53, 0x54, 0x55, 0x56, 0x57, 0x58, 0x59, 0x5a, /* A-Z */
+	0x61, 0x62, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68, 0x69, 0x6a, 0x6b, 0x6c, 0x6d, 0x6e, 0x6f,
+	0x70, 0x71, 0x72, 0x73, 0x74, 0x75, 0x76, 0x77, 0x78, 0x79, 0x7a, /* a-z */
+	0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39, /* 0-9 */
+	0x2b, /* + */
+	0x2f  /* / */
+};
+static const char utf8_b64_pad = 0x3d;
+
+static gboolean b64_decode_char (char c, int * b64)
+{
+	if ((c >= 0x41) && (c <= 0x5a))
+		{
+			*b64 = c - 0x41;
+			return TRUE;
+		}
+	if ((c >= 0x61) && (c <= 0x7a))
+		{
+			*b64 = c - (0x61 - 26);
+			return TRUE;
+		}
+	if ((c >= 0x30) && (c <= 0x39))
+		{
+			*b64 = c + (52 - 0x30);
+			return TRUE;
+		}
+	if (c == 0x2b)
+		{
+			*b64 = 62;
+			return TRUE;
+		}
+	if (c == 0x2f)
+		{
+			*b64 = 63;
+			return TRUE;
+		}
+	return FALSE;
+}
+
+static gboolean utf8_base64_decode(char ** binptr, size_t * binlen, const char * b64ptr, size_t b64len)
+{
+	gboolean decoded = TRUE;
+	gboolean padding = FALSE;
+	
+	int i = 0;
+	glong ucs4_len, j;
+
+	unsigned char byte1 = 0;
+	unsigned char byte2;
+	
+	gunichar ucs4, * ucs4_str;
+	
+	if (b64len == 0) 
+		return TRUE;
+	
+	if ((binptr == 0) || (b64ptr == 0)) 
+		return FALSE;
+	
+	ucs4_str = g_utf8_to_ucs4_fast(b64ptr, b64len, &ucs4_len);
+	
+	for(j = 0; j < ucs4_len; j++)
+		{
+			ucs4 = ucs4_str[j];
+			if ((ucs4 & 0x7f) == ucs4)
+				{
+					int b64;
+					char c = (char)(ucs4);
+
+					if (b64_decode_char (c, &b64))
+						{
+							if (padding || (*binlen == 0))
+								{
+									decoded = FALSE;
+									break;
+								}
+
+							switch (i)
+								{
+								case 0:
+									byte1 = (unsigned char)(b64) << 2;
+									i++;
+									break;
+								case 1:
+									byte2 = (unsigned char)(b64);
+									byte1 |= byte2 >> 4;
+									*(*binptr)++ = (char)(byte1);
+									(*binlen)--;
+									byte1 = (byte2 & 0x0f) << 4;
+									i++;
+									break;
+								case 2:
+									byte2 = (unsigned char)(b64);
+									byte1 |= byte2 >> 2;
+									*(*binptr)++ = (char)(byte1);
+									(*binlen)--;
+									byte1 = (byte2 & 0x03) << 6;
+									i++;
+									break;
+								default:
+									byte1 |= (unsigned char)(b64);
+									*(*binptr)++ = (char)(byte1);
+									(*binlen)--;
+									i = 0;
+									break;
+								}
+							
+							if (!decoded) 
+								break;
+
+							continue;
+						}
+					else if (c == utf8_b64_pad)
+						{
+							switch (i)
+								{
+								case 0:
+								case 1:
+									decoded = FALSE;
+									break;
+								case 2:
+									if (*binlen == 0) 
+										decoded = FALSE;
+									else
+										{
+											*(*binptr)++ = (char)(byte1);
+											(*binlen)--;
+											padding = TRUE;
+										}
+									i++;
+									break;
+								default:
+									if (!padding)
+										{
+											if (*binlen == 0) 
+												decoded = FALSE;
+											else
+												{
+													*(*binptr)++ = (char)(byte1);
+													(*binlen)--;
+													padding = TRUE;
+												}
+										}
+									i = 0;
+									break;
+								}
+							if (!decoded) 
+								break;
+
+							continue;
+						}
+				}
+			if (g_unichar_isspace (ucs4)) 
+				continue;
+
+			decoded = FALSE;
+			break;
+		}
+	return decoded;
+}
+
+static GdkPixbuf *
+rsvg_pixbuf_new_from_data_at_size (const char *data,
+								   int         width, 
+								   int         height,
+								   gboolean    keep_aspect_ratio,
+								   GError    **error)
+{
+	GdkPixbufLoader *loader;
+	GdkPixbuf       *pixbuf;
+	
+	char * buffer, *bufptr;
+	size_t buffer_len, buffer_max_len, data_len;
+
+	struct {
+		gint width;
+		gint height;
+		gboolean keep_aspect_ratio;
+	} info;
+	
+	g_return_val_if_fail (data != NULL, NULL);
+	g_return_val_if_fail (width > 0 && height > 0, NULL);
+
+	while (*data) if (*data++ == ',') break;
+
+	data_len = strlen(data);
+	
+	buffer_max_len = ((data_len >> 2) + 1) * 3;
+	buffer_len = buffer_max_len;
+	buffer = g_new(char, buffer_max_len);
+	bufptr = buffer;
+
+	if(!utf8_base64_decode(&bufptr, &buffer_len, data, data_len)) {
+		g_free(buffer);
+		return NULL;
+	}
+
+	buffer_len = buffer_max_len - buffer_len;
+
+	loader = gdk_pixbuf_loader_new ();
+	
+	info.width = width;
+	info.height = height;
+	info.keep_aspect_ratio = keep_aspect_ratio;
+	
+	g_signal_connect (loader, "size-prepared", G_CALLBACK (size_prepared_cb), &info);
+
+	if (!gdk_pixbuf_loader_write (loader, buffer, buffer_len, error)) {
+		gdk_pixbuf_loader_close (loader, NULL);
+		g_object_unref (loader);
+		return NULL;
+	}
+	
+	if (!gdk_pixbuf_loader_close (loader, error)) {
+		g_object_unref (loader);
+		return NULL;
+	}
+	
+	pixbuf = gdk_pixbuf_loader_get_pixbuf (loader);
+	
+	if (!pixbuf) {
+		g_object_unref (loader);
+		g_set_error (error,
+					 GDK_PIXBUF_ERROR,
+					 GDK_PIXBUF_ERROR_FAILED,
+					 _("Failed to load image: reason not known, probably a corrupt image."));
+		return NULL;
+	}
+	
+	g_object_ref (pixbuf);
+	
+	g_object_unref (loader);
+	
+	g_free(buffer);
+	
+	return pixbuf;
+}
+
+GdkPixbuf *
+rsvg_pixbuf_new_from_file_at_size (const char *filename,
+								   int         width, 
+								   int         height,
+								   gboolean    keep_aspect_ratio,
+								   GError    **error)
+{
+	GdkPixbufLoader *loader;
+	GdkPixbuf       *pixbuf;
+	
+	guchar buffer [4096];
+	int length;
+	FILE *f;
+	struct {
+		gint width;
+		gint height;
+		gboolean keep_aspect_ratio;
+	} info;
+	
+	g_return_val_if_fail (filename != NULL, NULL);
+	g_return_val_if_fail (width > 0 && height > 0, NULL);
+	
+	f = fopen (filename, "rb");
+	if (!f) {
+		g_set_error (error,
+					 G_FILE_ERROR,
+					 g_file_error_from_errno (errno),
+					 _("Failed to open file '%s': %s"),
+					 filename, g_strerror (errno));
+		return NULL;
+	}
+	
+	loader = gdk_pixbuf_loader_new ();
+	
+	info.width = width;
+	info.height = height;
+	info.keep_aspect_ratio = keep_aspect_ratio;
+	
+	g_signal_connect (loader, "size-prepared", G_CALLBACK (size_prepared_cb), &info);
+	
+	while (!feof (f)) {
+		length = fread (buffer, 1, sizeof (buffer), f);
+		if (length > 0)
+			if (!gdk_pixbuf_loader_write (loader, buffer, length, error)) {
+				gdk_pixbuf_loader_close (loader, NULL);
+				fclose (f);
+				g_object_unref (loader);
+				return NULL;
+			}
+	}
+	
+	fclose (f);
+	
+	if (!gdk_pixbuf_loader_close (loader, error)) {
+		g_object_unref (loader);
+		return NULL;
+	}
+	
+	pixbuf = gdk_pixbuf_loader_get_pixbuf (loader);
+	
+	if (!pixbuf) {
+		g_object_unref (loader);
+		g_set_error (error,
+					 GDK_PIXBUF_ERROR,
+					 GDK_PIXBUF_ERROR_FAILED,
+					 _("Failed to load image '%s': reason not known, probably a corrupt image file"),
+					 filename);
+		return NULL;
+	}
+	
+	g_object_ref (pixbuf);
+	
+	g_object_unref (loader);
+	
+	return pixbuf;
+}
+
 /* TODO 1: issue with affining alpha images - this is gdkpixbuf's fault...
  * TODO 2: issue with rotating images - do we want to rotate the whole
  *         canvas 2x to get this right, only to have #1 bite us?
@@ -1035,11 +1384,11 @@ rsvg_start_image (RsvgHandle *ctx, RsvgPropertyBag *atts)
 	double x = 0., y = 0., w = -1., h = -1.;
 	const char * href = NULL;
 	const char * klazz = NULL, * id = NULL, *value;
-	
+	gboolean has_alpha;
+
 	GdkPixbuf *img;
 	GError *err = NULL;
 	
-	gboolean has_alpha;
 	guchar *rgb = NULL;
 	int dest_rowstride;
 	double tmp_affine[6];
@@ -1078,43 +1427,56 @@ rsvg_start_image (RsvgHandle *ctx, RsvgPropertyBag *atts)
 	if (!state->visible)
 		return;
 
-	img = gdk_pixbuf_new_from_file (href, &err);
+	w *= state->affine[0];
+	h *= state->affine[3];
+
+	if(!strncmp(href, "data:", 5))
+		img = rsvg_pixbuf_new_from_data_at_size (href, w, h, FALSE, &err);
+	else
+		img = rsvg_pixbuf_new_from_file_at_size (href, w, h, FALSE, &err);
 	
 	if (!img)
 		{
 			if (err)
 				{
-					g_warning ("Couldn't load pixbuf (%s): %s\n", href, err->message);
+					g_warning ("Couldn't load image: %s\n", err->message);
 					g_error_free (err);
 				}
 			return;
 		}
-	
-	/* scale/resize the dest image */
-	art_affine_scale (tmp_affine, (double)w / (double)gdk_pixbuf_get_width (img),
-					  (double)h / (double)gdk_pixbuf_get_height (img));
-	art_affine_multiply (state->affine, tmp_affine, state->affine);
-	
+
 	has_alpha = gdk_pixbuf_get_has_alpha (img);
+	if(0 /* has_alpha */)
+		{
+			GdkPixbuf *tmp_pixbuf;
+			
+			tmp_pixbuf = gdk_pixbuf_add_alpha(img, FALSE, 0, 0, 0);
+			g_object_unref(img);
+			img = tmp_pixbuf;
+			has_alpha = TRUE;
+		}
+
 	dest_rowstride = (int)(w * (has_alpha ? 4 : 3) + 3) & ~3;
 	rgb = g_new (guchar, h * dest_rowstride);
 	
+	/* we handle scaling above. we handle translation below. we don't handle rotation very well at all */
+	tmp_affine[0] = tmp_affine[3] = 1;
+	tmp_affine[4] = tmp_affine[5] = 0;
+	tmp_affine[1] = state->affine[1];
+	tmp_affine[2] = state->affine[2];
+
 	if(has_alpha)
 		art_rgb_rgba_affine (rgb, 0, 0, w, h, dest_rowstride,
 							 gdk_pixbuf_get_pixels (img),
-							 gdk_pixbuf_get_width (img),
-							 gdk_pixbuf_get_height (img),
-							 gdk_pixbuf_get_rowstride (img),
-							 state->affine,
+							 w, h, dest_rowstride,
+							 tmp_affine,
 							 ART_FILTER_NEAREST,
 							 NULL);
 	else
 		art_rgb_affine (rgb, 0, 0, w, h, dest_rowstride,
 						gdk_pixbuf_get_pixels (img),
-						gdk_pixbuf_get_width (img),
-						gdk_pixbuf_get_height (img),
-						gdk_pixbuf_get_rowstride (img),
-						state->affine,
+						w, h, dest_rowstride,
+						tmp_affine,
 						ART_FILTER_NEAREST,
 						NULL);
 	
@@ -1126,13 +1488,17 @@ rsvg_start_image (RsvgHandle *ctx, RsvgPropertyBag *atts)
 			g_free (rgb);
 			return;
 		}
-	
+
+	rsvg_push_discrete_layer(ctx);
+
 	gdk_pixbuf_copy_area (img, 0, 0,
-						  gdk_pixbuf_get_width (img) * state->affine[0],
-						  gdk_pixbuf_get_height (img) * state->affine[3],
+						  gdk_pixbuf_get_width (img),
+						  gdk_pixbuf_get_height (img),
 						  ctx->pixbuf, 
-						  state->affine[4] + x,
+						  state->affine[4] + x, /* translate */
 						  state->affine[5] + y);
+
+	rsvg_pop_discrete_layer(ctx);
 	
 	g_object_unref (G_OBJECT (img));
 	g_free (rgb);
@@ -1197,41 +1563,13 @@ rsvg_start_use (RsvgHandle *ctx, RsvgPropertyBag *atts)
 							
 							if (!ctx->in_defs)
 								{
-									state = rsvg_state_current (ctx);
-									
-									rsvg_state_clone (state, &use->super.state);
-									if (ctx->n_state > 1)
-										rsvg_state_reinherit(state, &ctx->state[ctx->n_state - 2]);
-									
-									if (state->opacity != 0xff || state->filter)
-										rsvg_push_discrete_layer (ctx);
-									
-									
-									/* push the state stack */
-									if (ctx->n_state == ctx->n_state_max)
-										ctx->state = g_renew (RsvgState, ctx->state, 
-															  ctx->n_state_max <<= 1);
-									if (ctx->n_state)
-										rsvg_state_inherit (&ctx->state[ctx->n_state],
-															&ctx->state[ctx->n_state - 1]);
-									else
-										rsvg_state_init (ctx->state);
-									ctx->n_state++;
-									
-									rsvg_defs_drawable_draw (use->child, ctx);
-									
-									/* pop the state stack */
-									ctx->n_state--;
-									rsvg_state_finalize (&ctx->state[ctx->n_state]);
-									
-									if (state->opacity != 0xff || state->filter)
-										rsvg_pop_discrete_layer (ctx);
+									rsvg_defs_drawable_draw (&use->super, ctx, 1);
+									use->super.super.free(&use->super.super);
 									break;
 								}							
 							else
 								{
-									if (id != NULL)
-										rsvg_defs_set (ctx->defs, id, &use->super.super);
+									rsvg_defs_set (ctx->defs, id, &use->super.super);
 									
 									use->super.parent = (RsvgDefsDrawable *)ctx->current_defs_group;
 									if (use->super.parent != NULL)
@@ -1243,7 +1581,7 @@ rsvg_start_use (RsvgHandle *ctx, RsvgPropertyBag *atts)
 								}
 						}
 					default:
-						g_warning ("Unhandled defs entry/type %s %d\n", id, 
+						g_warning (_("Unhandled defs entry/type %s %d\n"), id, 
 								   parent->type);
 						return;
 					}
