@@ -29,10 +29,28 @@
 #include "rsvg-filter.h"
 #include "rsvg-css.h"
 #include "rsvg-styles.h"
+#include "rsvg-shapes.h"
+#include "rsvg-mask.h"
 
 #include <libart_lgpl/art_rgba.h>
 #include <libart_lgpl/art_affine.h>
 
+#define RSVG_DEFAULT_FONT "Times New Roman"
+
+static guint32
+rsvg_state_current_color (RsvgState * cur_state, RsvgState * parent_state)
+{
+	if (cur_state)
+		{
+			if (cur_state->has_current_color)
+				return cur_state->current_color;
+			else if (parent_state)
+				return parent_state->current_color;
+		}
+	
+	return 0;
+}
+
 gdouble
 rsvg_viewport_percentage (gdouble width, gdouble height)
 {
@@ -46,9 +64,9 @@ rsvg_state_init (RsvgState *state)
 	
 	art_affine_identity (state->affine);
 	art_affine_identity (state->personal_affine);
-	
+	state->mask = NULL;
 	state->opacity = 0xff;
-	state->fill = rsvg_paint_server_parse (NULL, "#000");
+	state->fill = rsvg_paint_server_parse (NULL, NULL, "#000", 0);
 	state->fill_opacity = 0xff;
 	state->stroke_opacity = 0xff;
 	state->stroke_width = 1;
@@ -60,7 +78,7 @@ rsvg_state_init (RsvgState *state)
 	state->backgroundnew = FALSE;
 	state->save_pixbuf = NULL;
 
-	state->font_family  = g_strdup ("Times New Roman");
+	state->font_family  = g_strdup (RSVG_DEFAULT_FONT);
 	state->font_size    = 12.0;
 	state->font_style   = PANGO_STYLE_NORMAL;
 	state->font_variant = PANGO_VARIANT_NORMAL;
@@ -123,9 +141,9 @@ rsvg_state_reinherit (RsvgState *dst, const RsvgState *src)
 	
 	if (!dst->has_fill_server)
 		{
+			rsvg_paint_server_ref (src->fill);
 			rsvg_paint_server_unref (dst->fill);
 			dst->fill = src->fill;
-			rsvg_paint_server_ref (dst->fill);
 		} 
 	if (!dst->has_fill_opacity)
 		dst->fill_opacity = src->fill_opacity;
@@ -133,9 +151,9 @@ rsvg_state_reinherit (RsvgState *dst, const RsvgState *src)
 		dst->fill_rule = src->fill_rule;
 	if (!dst->has_stroke_server)
 		{
+			rsvg_paint_server_ref (src->stroke);
 			rsvg_paint_server_unref (dst->stroke);
 			dst->stroke = src->stroke;
-			rsvg_paint_server_ref (dst->stroke);
 		} 
 	if (!dst->has_stroke_opacity)
 		dst->stroke_opacity = src->stroke_opacity;
@@ -147,10 +165,8 @@ rsvg_state_reinherit (RsvgState *dst, const RsvgState *src)
 		dst->cap = src->cap;
 	if (!dst->has_join)
 		dst->join = src->join;
-	if (!dst->has_visible)
-		dst->cap = src->cap;
 	if (!dst->has_stop_color)
-		dst->join = src->join;
+		dst->stop_color = src->stop_color;
 	if (!dst->has_stop_opacity)
 		dst->stop_opacity = src->stop_opacity;
 	if (!dst->has_visible)
@@ -172,10 +188,14 @@ rsvg_state_reinherit (RsvgState *dst, const RsvgState *src)
 	if (!dst->has_text_anchor)
 		dst->text_anchor = src->text_anchor;
 
-	if (!dst->has_font_family)
+	if (!dst->has_font_family) {
+		g_free(dst->font_family); /* font_family is always set to something */
 		dst->font_family = g_strdup (src->font_family);
-	if (!dst->has_lang)
+	}
+
+	if (!dst->has_lang) {
 		dst->lang = g_strdup (src->lang);
+	}
 
 	if (src->dash.n_dash > 0 && !dst->has_dash)
 		{
@@ -186,6 +206,83 @@ rsvg_state_reinherit (RsvgState *dst, const RsvgState *src)
 	art_affine_multiply (dst->affine, dst->personal_affine, src->affine); 
 }
 
+void
+rsvg_state_dominate (RsvgState *dst, const RsvgState *src)
+{
+	gint i;
+	
+	if (!dst->has_fill_server || src->has_fill_server)
+		{
+			rsvg_paint_server_ref (src->fill);
+			rsvg_paint_server_unref (dst->fill);
+			dst->fill = src->fill;
+		} 
+	if (!dst->has_fill_opacity || src->has_fill_opacity)
+		dst->fill_opacity = src->fill_opacity;
+	if (!dst->has_fill_rule || src->has_fill_rule)
+		dst->fill_rule = src->fill_rule;
+	if (!dst->has_stroke_server || src->has_stroke_server)
+		{
+			rsvg_paint_server_ref (src->stroke);
+			rsvg_paint_server_unref (dst->stroke);
+			dst->stroke = src->stroke;
+		} 
+	if (!dst->has_stroke_opacity || src->has_stroke_opacity)
+		dst->stroke_opacity = src->stroke_opacity;
+	if (!dst->has_stroke_width || src->has_stroke_width)
+		dst->stroke_width = src->stroke_width;
+	if (!dst->has_miter_limit || src->has_miter_limit)
+		dst->miter_limit = src->miter_limit;
+	if (!dst->has_cap || src->has_cap)
+		dst->cap = src->cap;
+	if (!dst->has_join || src->has_join)
+		dst->join = src->join;
+	if (!dst->has_stop_color || src->has_stop_color)
+		dst->stop_color = src->stop_color;
+	if (!dst->has_stop_opacity || src->has_stop_opacity)
+		dst->stop_opacity = src->stop_opacity;
+	if (!dst->has_visible || src->has_visible)
+		dst->visible = src->visible;
+	if (!dst->has_font_size || src->has_font_size)
+		dst->font_size = src->font_size;
+	if (!dst->has_font_style || src->has_font_style)
+		dst->font_style = src->font_style;
+	if (!dst->has_font_variant || src->has_font_variant)
+		dst->font_variant = src->font_variant;
+	if (!dst->has_font_weight || src->has_font_weight)
+		dst->font_weight = src->font_weight;
+	if (!dst->has_font_stretch || src->has_font_stretch)
+		dst->font_stretch = src->font_stretch;
+	if (!dst->has_font_decor || src->has_font_decor)
+		dst->font_decor = src->font_decor;
+	if (!dst->has_text_dir || src->has_text_dir)
+		dst->text_dir = src->text_dir;
+	if (!dst->has_text_anchor || src->has_text_anchor)
+		dst->text_anchor = src->text_anchor;
+
+	if (!dst->has_font_family || src->has_font_family) {
+		g_free(dst->font_family); /* font_family is always set to something */
+		dst->font_family = g_strdup (src->font_family);
+	}
+
+	if (!dst->has_lang || src->has_lang) {
+		if(dst->has_lang)
+			g_free(dst->lang); 
+		dst->lang = g_strdup (src->lang);
+	}
+
+	if (src->dash.n_dash > 0 && (!dst->has_dash || src->has_dash))
+		{
+			if(dst->has_dash)
+				g_free(dst->dash.dash);
+
+			dst->dash.dash = g_new (gdouble, src->dash.n_dash);
+			for (i = 0; i < src->dash.n_dash; i++)
+				dst->dash.dash[i] = src->dash.dash[i];
+		}
+	art_affine_multiply (dst->affine, dst->personal_affine, src->affine); 
+}
+
 void
 rsvg_state_inherit (RsvgState *dst, const RsvgState *src)
 {
@@ -209,16 +306,31 @@ rsvg_state_finalize (RsvgState *state)
 static void
 rsvg_parse_style_arg (RsvgHandle *ctx, RsvgState *state, const char *str)
 {
-	RsvgState * parent_state = state; /* TODO: temporary hack */
+	RsvgState * parent_state;
 	int arg_off;
 	
+	parent_state = rsvg_state_parent (ctx);
 	arg_off = rsvg_css_param_arg_offset (str);
-	if (rsvg_css_param_match (str, "opacity"))
+
+	if (rsvg_css_param_match (str, "color"))
+		{
+			RsvgState * parent_state = rsvg_state_parent (ctx);
+
+			if (parent_state)
+				state->current_color = rsvg_css_parse_color (str + arg_off, parent_state->current_color);
+			else
+				state->current_color = rsvg_css_parse_color (str + arg_off, 0);
+
+			state->has_current_color = TRUE;
+		}
+	else if (rsvg_css_param_match (str, "opacity"))
 		{
 			state->opacity = rsvg_css_parse_opacity (str + arg_off);
 		}
 	else if (rsvg_css_param_match (str, "filter"))
 		state->filter = rsvg_filter_parse(ctx->defs, str + arg_off);
+	else if (rsvg_css_param_match (str, "mask"))
+		state->mask = rsvg_mask_parse(ctx->defs, str + arg_off);
 	else if (rsvg_css_param_match (str, "enable-background"))
 		{
 			if (!strcmp (str + arg_off, "new"))
@@ -248,9 +360,17 @@ rsvg_parse_style_arg (RsvgHandle *ctx, RsvgState *state, const char *str)
 		}
 	else if (rsvg_css_param_match (str, "fill"))
 		{
-			rsvg_paint_server_unref (state->fill);
-			state->fill = rsvg_paint_server_parse (ctx->defs, str + arg_off);
+			RsvgPaintServer * fill = state->fill;
+			guint32 current_color = rsvg_state_current_color (state, parent_state);
+
+			if (parent_state)
+				state->fill = rsvg_paint_server_parse (parent_state->fill, ctx->defs, str + arg_off, current_color);
+			else
+				state->fill = rsvg_paint_server_parse (NULL, ctx->defs, str + arg_off, current_color);
+
 			state->has_fill_server = TRUE;
+
+			rsvg_paint_server_unref (fill);
 		}
 	else if (rsvg_css_param_match (str, "fill-opacity"))
 		{
@@ -269,9 +389,17 @@ rsvg_parse_style_arg (RsvgHandle *ctx, RsvgState *state, const char *str)
 		}
 	else if (rsvg_css_param_match (str, "stroke"))
 		{
-			rsvg_paint_server_unref (state->stroke);
-			state->stroke = rsvg_paint_server_parse (ctx->defs, str + arg_off);
+			RsvgPaintServer * stroke = state->stroke;
+			guint32 current_color = rsvg_state_current_color (state, parent_state);
+
+			if (parent_state)
+				state->stroke = rsvg_paint_server_parse (parent_state->stroke, ctx->defs, str + arg_off, current_color);
+			else
+				state->stroke = rsvg_paint_server_parse (NULL, ctx->defs, str + arg_off, current_color);
+
 			state->has_stroke_server = TRUE;		
+
+			rsvg_paint_server_unref (stroke);
 		}
 	else if (rsvg_css_param_match (str, "stroke-width"))
 		{
@@ -316,7 +444,8 @@ rsvg_parse_style_arg (RsvgHandle *ctx, RsvgState *state, const char *str)
 		}
 	else if (rsvg_css_param_match (str, "font-family"))
 		{
-			char * save = g_strdup (rsvg_css_parse_font_family (str + arg_off, parent_state->font_family));
+			char * save = g_strdup (rsvg_css_parse_font_family (str + arg_off, 
+																(parent_state ? parent_state->font_family : RSVG_DEFAULT_FONT)));
 			g_free (state->font_family);
 			state->font_family = save;
 			state->has_font_family = TRUE;
@@ -330,22 +459,26 @@ rsvg_parse_style_arg (RsvgHandle *ctx, RsvgState *state, const char *str)
 		}
 	else if (rsvg_css_param_match (str, "font-style"))
 		{
-			state->font_style = rsvg_css_parse_font_style (str + arg_off, parent_state->font_style);
+			state->font_style = rsvg_css_parse_font_style (str + arg_off, 
+														   (parent_state ? parent_state->font_style : PANGO_STYLE_NORMAL));
 			state->has_font_style = TRUE;
 		}
 	else if (rsvg_css_param_match (str, "font-variant"))
 		{
-			state->font_variant = rsvg_css_parse_font_variant (str + arg_off, parent_state->font_variant);
+			state->font_variant = rsvg_css_parse_font_variant (str + arg_off, 
+															   (parent_state ? parent_state->font_variant : PANGO_VARIANT_NORMAL));
 			state->has_font_variant = TRUE;
 		}
 	else if (rsvg_css_param_match (str, "font-weight"))
 		{
-			state->font_weight = rsvg_css_parse_font_weight (str + arg_off, parent_state->font_weight);
+			state->font_weight = rsvg_css_parse_font_weight (str + arg_off, 
+															 (parent_state ? parent_state->font_weight : PANGO_WEIGHT_NORMAL));
 			state->has_font_weight = TRUE;
 		}
 	else if (rsvg_css_param_match (str, "font-stretch"))
 		{
-			state->font_stretch = rsvg_css_parse_font_stretch (str + arg_off, parent_state->font_stretch);
+			state->font_stretch = rsvg_css_parse_font_stretch (str + arg_off, 
+															   (parent_state ? parent_state->font_stretch : PANGO_STRETCH_NORMAL));
 			state->has_font_stretch = TRUE;
 		}
 	else if (rsvg_css_param_match (str, "text-decoration"))
@@ -353,7 +486,7 @@ rsvg_parse_style_arg (RsvgHandle *ctx, RsvgState *state, const char *str)
 			if (!strcmp (str + arg_off, "inherit"))
 				{
 					state->has_font_decor = FALSE;
-					state->font_decor = parent_state->font_decor;
+					state->font_decor = (parent_state ? parent_state->font_decor : TEXT_NORMAL);
 				}
 			else 
 				{
@@ -372,7 +505,7 @@ rsvg_parse_style_arg (RsvgHandle *ctx, RsvgState *state, const char *str)
 			/* lr-tb | rl-tb | tb-rl | lr | rl | tb | inherit */
 			if (!strcmp (str + arg_off, "inherit"))
 				{
-					state->text_dir = parent_state->text_dir;
+					state->text_dir = (parent_state ? parent_state->text_dir : PANGO_DIRECTION_LTR);
 					state->has_text_dir = FALSE;
 				}
 			else if (!strcmp (str + arg_off, "rl"))
@@ -391,7 +524,7 @@ rsvg_parse_style_arg (RsvgHandle *ctx, RsvgState *state, const char *str)
 			state->has_text_anchor = TRUE;
 			if (!strcmp (str + arg_off, "inherit"))
 				{
-					state->text_anchor = parent_state->text_anchor;
+					state->text_anchor = (parent_state ? parent_state->text_anchor : TEXT_ANCHOR_START);
 					state->has_text_anchor = FALSE;
 				}
 			else 
@@ -406,8 +539,10 @@ rsvg_parse_style_arg (RsvgHandle *ctx, RsvgState *state, const char *str)
 		}
 	else if (rsvg_css_param_match (str, "stop-color"))
 		{
+			guint32 current_color = rsvg_state_current_color (state, parent_state);
+
 			state->has_stop_color = TRUE;
-			state->stop_color = rsvg_css_parse_color (str + arg_off);
+			state->stop_color = rsvg_css_parse_color (str + arg_off, current_color);
 		}
 	else if (rsvg_css_param_match (str, "stop-opacity"))
 		{
@@ -494,6 +629,7 @@ void
 rsvg_parse_style_pairs (RsvgHandle *ctx, RsvgState *state, 
 						RsvgPropertyBag *atts)
 {
+			rsvg_lookup_parse_style_pair (ctx, state, "color", atts);
 			rsvg_lookup_parse_style_pair (ctx, state, "display", atts);
 			rsvg_lookup_parse_style_pair (ctx, state, "enable-background", atts);
 			rsvg_lookup_parse_style_pair (ctx, state, "fill", atts);
@@ -506,6 +642,7 @@ rsvg_parse_style_pairs (RsvgHandle *ctx, RsvgState *state,
 			rsvg_lookup_parse_style_pair (ctx, state, "font-variant", atts);
 			rsvg_lookup_parse_style_pair (ctx, state, "font-weight", atts);
 			rsvg_lookup_parse_style_pair (ctx, state, "opacity", atts);
+			rsvg_lookup_parse_style_pair (ctx, state, "mask", atts);
 			rsvg_lookup_parse_style_pair (ctx, state, "filter", atts);
 			rsvg_lookup_parse_style_pair (ctx, state, "stop-color", atts);
 			rsvg_lookup_parse_style_pair (ctx, state, "stop-opacity", atts);
@@ -701,8 +838,9 @@ rsvg_real_parse_cssbuffer (RsvgHandle *ctx, const char * buff, size_t buflen)
     
 	if (status != CR_OK)
         {
-			g_warning ("Error setting CSS SAC handler\n");
-			cr_parser_destroy (parser);			
+			g_warning (_("Error setting CSS SAC handler"));
+			cr_parser_destroy (parser);
+			cr_doc_handler_destroy (css_handler);
 			return;
         }        
 	
@@ -710,6 +848,7 @@ rsvg_real_parse_cssbuffer (RsvgHandle *ctx, const char * buff, size_t buflen)
 	status = cr_parser_parse (parser);
 	
 	cr_parser_destroy (parser);
+	cr_doc_handler_destroy (css_handler);
 }
 
 #else /* !HAVE_LIBCROCO */
@@ -1080,7 +1219,7 @@ rsvg_push_discrete_layer (RsvgHandle *ctx)
 	pixbuf = ctx->pixbuf;
 	
 	if (state->filter == NULL && state->opacity == 0xFF && 
-		!state->backgroundnew)
+		!state->backgroundnew && state->mask == NULL)
 		return;
 
 	state->save_pixbuf = pixbuf;
@@ -1093,7 +1232,7 @@ rsvg_push_discrete_layer (RsvgHandle *ctx)
 
 	if (!gdk_pixbuf_get_has_alpha (pixbuf))
     {
-		g_warning ("push/pop transparency group on non-alpha buffer nyi");
+		g_warning (_("push/pop transparency group on non-alpha buffer nyi"));
 		return;
     }
 	
@@ -1115,14 +1254,6 @@ rsvg_push_discrete_layer (RsvgHandle *ctx)
 	ctx->pixbuf = pixbuf;
 }
 
-/**
- * rsvg_pop_opacity_group: End a transparency group.
- * @ctx: Context in which to push.
- * @opacity: Opacity for blending (0..255).
- *
- * Pops a new transparency group from the stack, recompositing with the
- * next on stack.
- **/
 static void
 rsvg_use_opacity (RsvgHandle *ctx, int opacity, 
 				  GdkPixbuf *tos, GdkPixbuf *nos)
@@ -1143,7 +1274,7 @@ rsvg_use_opacity (RsvgHandle *ctx, int opacity,
 	
 	if (!gdk_pixbuf_get_has_alpha (nos))
 		{
-			g_warning ("push/pop transparency group on non-alpha buffer nyi");
+			g_warning (_("push/pop transparency group on non-alpha buffer nyi"));
 			return;
 		}
 	
@@ -1175,17 +1306,6 @@ rsvg_use_opacity (RsvgHandle *ctx, int opacity,
 		}
 }
 
-/**
- * rsvg_pop_opacity_group_as_filter: End a transparency group using a filter.
- * @ctx: Context in which to push.
- * @filter: A pointer to the filter to be used.
- *
- * Pops a new transparency group from the stack, recompositing with the
- * next on stack using a filter to do so.
- * It is a little bit of a hack to use this stacks for filters too, but it the
- * same principle in general
- **/
-
 static GdkPixbuf *
 get_next_out(gint * operationsleft, GdkPixbuf * in, GdkPixbuf * tos, 
 			 GdkPixbuf * nos, GdkPixbuf *intermediate)
@@ -1207,10 +1327,6 @@ get_next_out(gint * operationsleft, GdkPixbuf * in, GdkPixbuf * tos,
 	return out;
 }
 
-static void
-rsvg_composite_layer(RsvgHandle *ctx, RsvgState *state, GdkPixbuf *tos, 
-					 GdkPixbuf *nos);
-
 static GdkPixbuf *
 rsvg_compile_bg(RsvgHandle *ctx, RsvgState *topstate)
 {
@@ -1253,6 +1369,7 @@ rsvg_composite_layer(RsvgHandle *ctx, RsvgState *state, GdkPixbuf *tos, GdkPixbu
 {
 	RsvgFilter *filter = state->filter;
 	int opacity = state->opacity;
+	RsvgDefsDrawable * mask = state->mask;
 	GdkPixbuf *intermediate;
 	GdkPixbuf *in, *out, *insidebg;
 	int operationsleft;
@@ -1260,7 +1377,7 @@ rsvg_composite_layer(RsvgHandle *ctx, RsvgState *state, GdkPixbuf *tos, GdkPixbu
 	intermediate = NULL;
 
 	if (state->filter == NULL && state->opacity == 0xFF && 
-		!state->backgroundnew)
+		!state->backgroundnew && state->mask == NULL)
 		return;
 
 	operationsleft = 0;
@@ -1269,6 +1386,8 @@ rsvg_composite_layer(RsvgHandle *ctx, RsvgState *state, GdkPixbuf *tos, GdkPixbu
 		operationsleft++;
 	if (filter != NULL)
 		operationsleft++;
+	if (mask != NULL)
+		operationsleft++;
 
 	if (operationsleft > 1)
 		intermediate = gdk_pixbuf_new (GDK_COLORSPACE_RGB, 1, 8, 
@@ -1296,12 +1415,26 @@ rsvg_composite_layer(RsvgHandle *ctx, RsvgState *state, GdkPixbuf *tos, GdkPixbu
 			rsvg_use_opacity (ctx, opacity, in, out);
 			in = out;
 		}
+	if (mask != NULL)
+		{
+			out = get_next_out(&operationsleft, in, tos, nos, intermediate);
+			rsvg_mask_render ((RsvgMask *)mask, in, out, ctx);
+			in = out;
+		}
 
 	if (intermediate != NULL)
 		g_object_unref (intermediate);
 
 }
 
+/**
+ * rsvg_pop_discrete_layer: End a transparency group.
+ * @ctx: Context in which to push.
+ *
+ * Pops a new transparency group from the stack, recompositing with the
+ * next on stack using a filter, transperency value, or a mask to do so
+ **/
+
 void
 rsvg_pop_discrete_layer(RsvgHandle *ctx)
 {
@@ -1310,15 +1443,15 @@ rsvg_pop_discrete_layer(RsvgHandle *ctx)
 
 	state = rsvg_state_current(ctx);
 
-	if (state->filter == NULL && 
-		state->opacity == 0xFF && 
-		!state->backgroundnew)
+	if (state->filter == NULL && state->opacity == 0xFF && 
+		!state->backgroundnew && state->mask == NULL)
 		return;
 
 	tos = ctx->pixbuf;
 	nos = state->save_pixbuf;
 	
-	rsvg_composite_layer(ctx, state, tos, nos);
+	if (nos != NULL)
+		rsvg_composite_layer(ctx, state, tos, nos);
 	
 	g_object_unref (tos);
 	ctx->pixbuf = nos;
@@ -1332,6 +1465,14 @@ rsvg_state_current (RsvgHandle *ctx)
 	return NULL;
 }
 
+RsvgState *
+rsvg_state_parent (RsvgHandle *ctx)
+{
+	if (ctx->n_state > 1)
+		return &ctx->state[ctx->n_state - 2];
+	return NULL;
+}
+
 double
 rsvg_state_current_font_size (RsvgHandle *ctx)
 {
