@@ -70,6 +70,7 @@ struct _RsvgState {
 	gboolean has_fill_rule;
 
 	RsvgFilter *filter;
+	void *mask;
 	gboolean backgroundnew;
 
 	RsvgPaintServer *stroke;
@@ -119,7 +120,10 @@ struct _RsvgState {
 
 	ArtVpathDash dash;
 	gboolean has_dash;
-	
+
+	guint32 current_color;
+	gboolean has_current_color;
+
 	GdkPixbuf *save_pixbuf;
 };
 
@@ -127,6 +131,7 @@ void rsvg_state_init (RsvgState *state);
 void rsvg_state_clone (RsvgState *dst, const RsvgState *src);
 void rsvg_state_inherit (RsvgState *dst, const RsvgState *src);
 void rsvg_state_reinherit (RsvgState *dst, const RsvgState *src);
+void rsvg_state_dominate (RsvgState *dst, const RsvgState *src);
 void rsvg_state_finalize (RsvgState *state);
 
 void rsvg_parse_style_pairs (RsvgHandle *ctx, RsvgState *state, 
@@ -146,6 +151,7 @@ rsvg_pop_discrete_layer(RsvgHandle *ctx);
 void rsvg_push_discrete_layer (RsvgHandle *ctx);
 gboolean rsvg_parse_transform (double dst[6], const char *src);
 
+RsvgState * rsvg_state_parent (RsvgHandle *ctx);
 RsvgState * rsvg_state_current (RsvgHandle *ctx);
 double rsvg_state_current_font_size (RsvgHandle *ctx);
 
