@@ -30,8 +30,8 @@
 
 G_BEGIN_DECLS
 
-void rsvg_start_text (RsvgHandle *ctx, const xmlChar **atts);
-void rsvg_start_tspan (RsvgHandle *ctx, const xmlChar **atts);
+void rsvg_start_text (RsvgHandle *ctx, RsvgPropertyBag *atts);
+void rsvg_start_tspan (RsvgHandle *ctx, RsvgPropertyBag *atts);
 char * make_valid_utf8 (const char *str);
 
 void 
