@@ -32,6 +32,7 @@
 #include "rsvg-shapes.h"
 #include "rsvg-text.h"
 #include "rsvg-filter.h"
+#include "rsvg-mask.h"
 
 #include <math.h>
 #include <string.h>
@@ -76,9 +77,7 @@ rsvg_start_svg (RsvgHandle *ctx, RsvgPropertyBag *atts)
 	int width = -1, height = -1, x = -1, y = -1;
 	int rowstride;
 	art_u8 *pixels;
-	gint percent, em, ex;
 	RsvgState *state;
-	gboolean has_alpha = TRUE;
 	gint new_width, new_height;
 	double x_zoom = 1.;
 	double y_zoom = 1.;
@@ -92,19 +91,19 @@ rsvg_start_svg (RsvgHandle *ctx, RsvgPropertyBag *atts)
 		{
 			/* x & y should be ignored since we should always be the outermost SVG,
 			   at least for now, but i'll include them here anyway */
-			if ((value = rsvg_property_bag_lookup (atts, "width")))
-				width = rsvg_css_parse_length (value, ctx->dpi, &percent, &em, &ex);
-			if ((value = rsvg_property_bag_lookup (atts, "height")))
-				height = rsvg_css_parse_length (value, ctx->dpi, &percent, &em, &ex);
-			if ((value = rsvg_property_bag_lookup (atts, "x")))
-				x = rsvg_css_parse_length (value, ctx->dpi, &percent, &em, &ex);
-			if ((value = rsvg_property_bag_lookup (atts, "y")))
-				y = rsvg_css_parse_length (value, ctx->dpi, &percent, &em, &ex);
 			if ((value = rsvg_property_bag_lookup (atts, "viewBox")))
 				{
 					has_vbox = rsvg_css_parse_vbox (value, &vbox_x, &vbox_y,
 													&vbox_w, &vbox_h);
 				}
+			if ((value = rsvg_property_bag_lookup (atts, "width")))
+				width = rsvg_css_parse_normalized_length (value, ctx->dpi, vbox_w, 1);
+			if ((value = rsvg_property_bag_lookup (atts, "height")))
+				height = rsvg_css_parse_normalized_length (value, ctx->dpi, vbox_h, 1);
+			if ((value = rsvg_property_bag_lookup (atts, "x")))
+				x = rsvg_css_parse_normalized_length (value, ctx->dpi, vbox_w, 1);
+			if ((value = rsvg_property_bag_lookup (atts, "y")))
+				y = rsvg_css_parse_normalized_length (value, ctx->dpi, vbox_h, 1);
 	
 			if (has_vbox && vbox_w > 0. && vbox_h > 0.)
 				{
@@ -122,6 +121,14 @@ rsvg_start_svg (RsvgHandle *ctx, RsvgPropertyBag *atts)
 					new_width  = width;
 					new_height = height;
 
+					/* bogus hack */
+					if (new_width <= 0 || new_height <= 0)
+						{
+							g_warning (_("rsvg_start_svg: width and height not specified in the SVG"));
+							if (new_width <= 0) {width = new_width = 512;}
+							if (new_height <= 0) {height = new_height = 512;}
+						}
+
 					/* apply the sizing function to acquire our new width and height.
 					   we'll scale this against the old values later */
 					if (ctx->size_func)
@@ -161,24 +168,24 @@ rsvg_start_svg (RsvgHandle *ctx, RsvgPropertyBag *atts)
 			art_affine_scale (affine, x_zoom, y_zoom);
 			art_affine_multiply (state->affine, state->affine, affine);
 
-			if (new_width < 0 || new_height < 0)
+			if (new_width <= 0 || new_height <= 0)
 				{
-					g_warning ("rsvg_start_svg: width and height not specified in the SVG, nor supplied by the size callback");
-					if (new_width < 0) new_width = 500;
-					if (new_height < 0) new_height = 500;
+					g_warning (_("rsvg_start_svg: width and height not specified in the SVG, nor supplied by the size callback"));
+					if (new_width <= 0) new_width = 512;
+					if (new_height <= 0) new_height = 512;
 				}
 
 			if (new_width >= INT_MAX / 4)
 				{
 					/* FIXME: GError here? */
-					g_warning ("rsvg_start_svg: width too large");
+					g_warning (_("rsvg_start_svg: width too large"));
 					return;
 				}
-			rowstride = (new_width * (has_alpha ? 4 : 3) + 3) & ~3;
+			rowstride = (new_width * 4 + 3) & ~3;
 			if (rowstride > INT_MAX / new_height)
 				{
 					/* FIXME: GError here? */
-					g_warning ("rsvg_start_svg: width too large");
+					g_warning (_("rsvg_start_svg: width too large"));
 					return;
 				}
 
@@ -188,13 +195,13 @@ rsvg_start_svg (RsvgHandle *ctx, RsvgPropertyBag *atts)
 			if (pixels == NULL)
 				{
 					/* FIXME: GError here? */
-					g_warning ("rsvg_start_svg: dimensions too large");
+					g_warning (_("rsvg_start_svg: dimensions too large"));
 					return;
 				}
-			memset (pixels, has_alpha ? 0 : 255, rowstride * new_height);
+			memset (pixels, 0, rowstride * new_height);
 			ctx->pixbuf = gdk_pixbuf_new_from_data (pixels,
 													GDK_COLORSPACE_RGB,
-													has_alpha, 8,
+													TRUE, 8,
 													new_width, new_height,
 													rowstride,
 													rsvg_pixmap_destroy,
@@ -302,7 +309,7 @@ rsvg_gradient_stop_handler_start (RsvgSaxHandler *self, const xmlChar *name,
 	
 	if (!got_offset)
 		{
-			g_warning ("gradient stop must specify offset\n");
+			g_warning (_("gradient stop must specify offset\n"));
 			return;
 		}
 	
@@ -404,19 +411,19 @@ rsvg_start_linear_gradient (RsvgHandle *ctx, RsvgPropertyBag *atts)
 			if ((value = rsvg_property_bag_lookup (atts, "id")))
 				id = value;
 			if ((value = rsvg_property_bag_lookup (atts, "x1"))) {
-				x1 = rsvg_css_parse_normalized_length (value, ctx->dpi, (gdouble)ctx->width, state->font_size);
+				x1 = rsvg_css_parse_normalized_length (value, ctx->dpi, 1, state->font_size);
 				got_x1 = TRUE;
 			}
 			if ((value = rsvg_property_bag_lookup (atts, "y1"))) {
-				y1 = rsvg_css_parse_normalized_length (value, ctx->dpi, (gdouble)ctx->height, state->font_size);
+				y1 = rsvg_css_parse_normalized_length (value, ctx->dpi, 1, state->font_size);
 				got_y1 = TRUE;
 			}
 			if ((value = rsvg_property_bag_lookup (atts, "x2"))) {
-				x2 = rsvg_css_parse_normalized_length (value, ctx->dpi, (gdouble)ctx->width, state->font_size);
+				x2 = rsvg_css_parse_normalized_length (value, ctx->dpi, 1, state->font_size);
 				got_x2 = TRUE;
 			}
 			if ((value = rsvg_property_bag_lookup (atts, "y2"))) {
-				y2 = rsvg_css_parse_normalized_length (value, ctx->dpi, (gdouble)ctx->height, state->font_size);
+				y2 = rsvg_css_parse_normalized_length (value, ctx->dpi, 1, state->font_size);
 				got_y2 = TRUE;
 			}
 			if ((value = rsvg_property_bag_lookup (atts, "spreadMethod")))
@@ -521,25 +528,24 @@ rsvg_start_radial_gradient (RsvgHandle *ctx, RsvgPropertyBag *atts, const char *
 			if ((value = rsvg_property_bag_lookup (atts, "id")))
 				id = value;
 			if ((value = rsvg_property_bag_lookup (atts, "cx"))) {
-				cx = rsvg_css_parse_normalized_length (value, ctx->dpi, (gdouble)ctx->width, state->font_size);
+				cx = rsvg_css_parse_normalized_length (value, ctx->dpi, 1, state->font_size);
 				got_cx = TRUE;
 			}
 			if ((value = rsvg_property_bag_lookup (atts, "cy"))) {
-				cy = rsvg_css_parse_normalized_length (value, ctx->dpi, (gdouble)ctx->height, state->font_size);
+				cy = rsvg_css_parse_normalized_length (value, ctx->dpi, 1, state->font_size);
 				got_cy = TRUE;
 			}
 			if ((value = rsvg_property_bag_lookup (atts, "r"))) {
-				r = rsvg_css_parse_normalized_length (value, ctx->dpi, 
-													  rsvg_viewport_percentage((gdouble)ctx->width, (gdouble)ctx->height), 
+				r = rsvg_css_parse_normalized_length (value, ctx->dpi, 1, 
 													  state->font_size);
 				got_r = TRUE;
 			}
 			if ((value = rsvg_property_bag_lookup (atts, "fx"))) {
-				fx = rsvg_css_parse_normalized_length (value, ctx->dpi, (gdouble)ctx->width, state->font_size);
+				fx = rsvg_css_parse_normalized_length (value, ctx->dpi, 1, state->font_size);
 				got_fx = TRUE;
 			}
 			if ((value = rsvg_property_bag_lookup (atts, "fy"))) {
-				fy = rsvg_css_parse_normalized_length (value, ctx->dpi, (gdouble)ctx->height, state->font_size);
+				fy = rsvg_css_parse_normalized_length (value, ctx->dpi, 1, state->font_size);
 				got_fy = TRUE;
 			}
 			if ((value = rsvg_property_bag_lookup (atts, "xlink:href")))
@@ -786,7 +792,9 @@ rsvg_defs_handler_start (RsvgSaxHandler *self, const xmlChar *name,
 	 * onscreen representation of the colour wrong than not have any colour displayed whatsoever
 	 */
 
-	if (!strcmp ((char *)name, "linearGradient"))
+	if (!strcmp ((char *)name, "defs"))
+		ctx->in_defs++;
+	else if (!strcmp ((char *)name, "linearGradient"))
 		rsvg_start_linear_gradient (ctx, atts);
 	else if (!strcmp ((char *)name, "radialGradient"))
 		rsvg_start_radial_gradient (ctx, atts, "radialGradient");
@@ -812,7 +820,8 @@ rsvg_defs_handler_start (RsvgSaxHandler *self, const xmlChar *name,
 		rsvg_start_polygon (ctx, atts);
 	else if (!strcmp ((char *)name, "polyline"))
 		rsvg_start_polyline (ctx, atts);
-	
+	else if (!strcmp ((char *)name, "mask"))
+		rsvg_start_mask(ctx, atts);	
 	rsvg_filter_handler_start (ctx, name, atts);
 }
 
@@ -829,13 +838,15 @@ rsvg_defs_handler_end (RsvgSaxHandler *self, const xmlChar *name)
 					ctx->handler->free (ctx->handler);
 					ctx->handler = NULL;
 				}
-			ctx->in_defs = FALSE;
+			ctx->in_defs--;
 		}
 
 	if (!strcmp ((char *)name, "g"))
 		rsvg_end_g (ctx);
 	if (!strcmp ((char *)name, "filter"))
 		rsvg_end_filter (ctx);
+	else if (!strcmp ((char *)name, "mask"))
+		rsvg_end_mask(ctx);
 	
 	/* pop the state stack */
 	ctx->n_state--;
@@ -853,7 +864,7 @@ rsvg_start_defs (RsvgHandle *ctx, RsvgPropertyBag *atts)
 	handler->super.end_element   = rsvg_defs_handler_end;
 	handler->ctx = ctx;
 
-	ctx->in_defs = TRUE;	
+	ctx->in_defs++;	
 	ctx->handler = &handler->super;
 }
 
@@ -1074,6 +1085,11 @@ rsvg_start_element (void *data, const xmlChar *name, const xmlChar **atts)
 				rsvg_start_title (ctx, bag);
 			else if (!strcmp ((char *)name, "desc"))
 				rsvg_start_desc (ctx, bag);
+			else if (!strcmp ((char *)name, "mask"))
+				{
+					ctx->in_defs++;
+					rsvg_start_mask(ctx, bag);
+				}
 			
 			/* see conicalGradient discussion above */
 			else if (!strcmp ((char *)name, "linearGradient"))
@@ -1113,9 +1129,12 @@ rsvg_end_element (void *data, const xmlChar *name)
 			if (!strcmp ((char *)name, "filter"))
 				rsvg_end_filter (ctx);
 			else if (!strcmp ((char *)name, "defs")) {
-				ctx->in_defs = FALSE;
+				ctx->in_defs--;	
+			}
+			else if (!strcmp ((char *)name, "mask")){
+				rsvg_end_mask(ctx);
+				ctx->in_defs--;				
 			}
-			
 			/* pop the state stack */
 			ctx->n_state--;
 			rsvg_state_finalize (&ctx->state[ctx->n_state]);
